;;  Leopard Print - This is a script for The GIMP to generate a leopard print pattern
;;  Copyright (C) 2010  William Morrison
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define (script-fu-leopard-print img size)
 (cond
 ((gimp-procedural-db-proc-exists "script-fu-slime") ; dependency checks
 (let* ((new-layer 0))

  (gimp-image-undo-group-start img)
;; Generate the base for the pattern with the slime script
  (script-fu-slime img (* size 2))

;; Playing with levels and colours to get the final pattern
  (set! new-layer (car (gimp-image-get-active-layer img)))
  (gimp-equalize new-layer 1)
  (gimp-levels new-layer 0 32 128 1.00 0 255)
  (gimp-colorize new-layer 45 70 -50)
  
  (gimp-image-undo-group-end img)

  ; Update the display
  (gimp-displays-flush)
 )
 )
 (else (gimp-message "A dependency error occured in the script. You must have the slime script installed. (http://se2014.com/~wmorris/projects.html)"))
 )
)

(script-fu-register "script-fu-leopard-print"
                    "Leopard Print..."
                    "Generates a layer with a leopard print pattern. Depends on the slime filter."
                    "Will Morrison"
                    "GNU General Public License"
                    "2010"
                    "RGB*"
                    SF-IMAGE    "Image"         0
		    SF-ADJUSTMENT "Size" '(5 1 25 1 5 0 0))

(script-fu-menu-register "script-fu-leopard-print"
                         "<Image>/Filters/Will's Script Pack")
