;;  Psychedelic Lightning - This is a script for The GIMP to generate a spiky lightning pattern
;;  Copyright (C) 2010  William Morrison
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define (script-fu-crazy-lightning img iterations width height)
 (let* ((new-layer1 0) (new-layer2 0))

  (gimp-image-undo-group-start img)
 
;; Create the base layer and fill it with the solid noise filter
  (set! new-layer1 (car (gimp-layer-new img 1 1 0 "Lightning Layer" 100 0)))
  (gimp-image-add-layer img new-layer1 -1)
  (gimp-layer-resize-to-image-size new-layer1)
  (plug-in-solid-noise 1 img new-layer1 1 1 (random 65535) 2 width height)

;; Repeat adding a layer in difference mode, filling it with solid noise, and merging down
  (while (> iterations 0) 
    (set! new-layer2 (car (gimp-layer-copy new-layer1 1)))
    (gimp-image-add-layer img new-layer2 -1)
    (gimp-layer-set-mode new-layer2 6)
    (plug-in-solid-noise 1 img new-layer2 1 1 (random 65535) 1 width height)
    (set! new-layer1 (car (gimp-image-merge-down img new-layer2 2)))
    (set! iterations (- iterations 1))
  )

;; Playing with levels and another layer to achieve the end result
  (gimp-levels new-layer1 0 0 128 1.5 0 255)
  (set! new-layer2 (car (gimp-layer-copy new-layer1 1)))
  (gimp-image-add-layer img new-layer2 -1)
  (gimp-invert new-layer2)
  (gimp-layer-set-mode new-layer2 6)
  (set! new-layer1 (car (gimp-image-merge-down img new-layer2 2)))
  
  (gimp-image-undo-group-end img)

  ; Flush the display
  (gimp-displays-flush)

 )
)

(script-fu-register "script-fu-crazy-lightning"
                    "Psychedelic Lightning..."
                    "Adds a new layer and fills it with tileable lightning"
                    "Will Morrison"
                    "GNU General Public License"
                    "2010"
                    "RGB*"
                    SF-IMAGE    "Image"         0
                    SF-ADJUSTMENT "Number of iterations" '(5 1 20 1 5 0 0)
		    SF-ADJUSTMENT "X size" '(8 0.1 16 0.1 2 0 0)
		    SF-ADJUSTMENT "Y size" '(8 0.1 16 0.1 2 0 0)
)
(script-fu-menu-register "script-fu-crazy-lightning"
                         "<Image>/Filters/Will's Script Pack")



