;;  Maze - This is a script for The GIMP to generate a maze pattern
;;  Copyright (C) 2010  William Morrison
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.

(define (color=? clr1 clr2) ;(list 1))
  (cond 
    ((not (and (list? clr1) (list? clr2))) (list 0))
    ((or (null? clr1) (null? clr2)) (list 1))
    ((= (car clr1) (car clr2)) (color=? (cdr clr1) (cdr clr2))) 
    (else (list 0))
  )
)

(define (whichdir img layer x y dir)
  (cond
         ((= dir 1) (deadend? img layer (+ x 1) y))
         ((= dir 2) (deadend? img layer (- x 1) y))
         ((= dir 3) (deadend? img layer x (+ y 1)))
         ((= dir 4) (deadend? img layer x (- y 1)))
         (else 1)
  )
)

(define (deadend? img layer x y); 0)
  (let* ((count 0) (dir 0))
    (cond
    ((or (< (min x y) 1) (> (max x y) 97)) 2)
    ((color=? '(255 255 255) (car (gimp-image-pick-color img layer x y 1 1 0.1)))
      (cond
	((= 1 (car (color=? '(255 255 255) (car (gimp-image-pick-color img layer (+ x 1) y 1 1 0.1)))))
	(set! count (+ count 1))
	(set! dir 1))
	(else 0)
      )
      (cond
        ((= 1 (car (color=? '(255 255 255) (car (gimp-image-pick-color img layer (- x 1) y 1 1 0.1)))))
        (set! count (+ count 1))
        (set! dir 2))
        (else 0)
      )
      (cond
        ((= 1 (car (color=? '(255 255 255) (car (gimp-image-pick-color img layer x (+ y 1) 1 1 0.1)))))
        (set! count (+ count 1))
        (set! dir 3))
        (else 0)
      )
      (cond
        ((= 1 (car (color=? '(255 255 255) (car (gimp-image-pick-color img layer x (- y 1) 1 1 0.1)))))
        (set! count (+ count 1))
        (set! dir 4))
        (else 0)
      )
      
      (cond
        ((< count 2)
        (gimp-rect-select img x y 1 1 2 0 0)
        (gimp-edit-fill layer 1)
	(max 1 (whichdir img layer x y dir))
     	) 
	(else 0)
      )
    )
    (else 0)
  ))
)

(define (solve img layer)
 (let* ((x 1) (y 1) (minc 1) (maxc 98) (numends 2) (end? 0) (result 1))
  (gimp-image-undo-freeze img)

  (while (> numends 0)
    (set! y minc)
    (while (< y maxc)
      (set! x minc)
      (while (< x maxc)
	(set! end? (deadend? img layer x y))
	(cond 
	  ((= 1 end?) (list 1))
	  ((= 2 end?) (set! x 99) (set! y 99) (set! numends 0) (set! result 0))
	  (else (list 0)) 
	)
	(set! x (+ x 2))
      )
    (set! y (+ y 2))
    )
  (set! numends (- numends 1))
  )
 (gimp-image-undo-thaw img)
 (list result)
 )
)

(define (script-fu-mazegen scale)
  (let* ((img 0) (layer 0) (solution 0) (count 0))

  (set! img (car (gimp-image-new 99 99 0)))

  (gimp-image-undo-group-start img)

  (gimp-context-set-foreground '(255 255 255))
  (gimp-context-set-background '(0 0 0))

  (set! layer (car (gimp-layer-new img 99 99 0 "Maze" 100 0)))
  (gimp-image-add-layer img layer -1)

  (plug-in-maze 1 img layer 1 1 0 0 (rand 2100000000) 57 1)
  (gimp-rect-select img 0 1 1 1 0 0 0)
  (gimp-rect-select img 98 97 1 1 0 0 0)
  (gimp-edit-fill layer 2)
  (gimp-selection-none img)

  (plug-in-rotate 1 img layer 2 0)

  (while (= count 0)
  (set! solution (car (gimp-layer-new-from-drawable layer img)))
  (gimp-image-add-layer img solution -1)
  (gimp-layer-set-mode solution 3)
  (set! count (car (solve img solution)))
  (cond
    ((= count 0) (gimp-message "Unsucessful solution attempt") (gimp-image-remove-layer img solution))
    (else (gimp-message "Solution found"))
  )
  )

  (gimp-image-scale-full img (* 99 (+ 1 (* scale 2))) (* 99 (+ 1 (* scale 2))) 0)

  (gimp-by-color-select layer '(255 255 255) 0 2 0 0 0 0);select all white colours
  (set! count scale)
  (while (> count 0)
  (gimp-selection-grow img 1)
  (set! count (- count 1))
  )
  (gimp-edit-fill layer 2)
  (gimp-selection-none img)

  (gimp-invert solution)

  (gimp-by-color-select solution '(255 255 255) 0 2 0 0 0 0);select all white colours
  (set! count (- scale 1))
  (while (> count 0)
  (gimp-selection-grow img 1)
  (set! count (- count 1))
  )
  (gimp-edit-fill solution 2)
  (gimp-selection-none img)

  (gimp-colorize solution 0 100 50)

 (gimp-drawable-set-visible solution 0)

  (gimp-display-new img)

  (gimp-image-undo-group-end img)

  ; Update the display
  (gimp-displays-flush)
  (list img)
 )
)

(script-fu-register "script-fu-mazegen"
                    "Maze Generator"
                    "Generates a new image with a solved maze in it. Takes a few minutes to run"
                    "Will Morrison"
                    "GNU General Public License"
                    "2010"
                    ""
	               SF-ADJUSTMENT "Scaling factor" '(5 0 10 1 2 0 0)
)
(script-fu-menu-register "script-fu-mazegen"
                         "<Image>/Filters/Will's Script Pack")
